<?php

namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\SM\Factory\Sm00029000LocalAtendimentoFactory as LocalAtendFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Teknisa\SM\Service\ParceiroNegocioService;
use Teknisa\SM\Service\EnderecoParceiroService;
use Teknisa\Model\Geral\Entities\Enderecoparc;
use Teknisa\SM\Factory\Sm00005000ParametrizacoesServicesFactory;
use Teknisa\SM\Service\IntegracoesTeknisaService;

class Sm00029000LocalAtendimentoService {
    
    protected $entityManager;
    protected $connection;
    protected $LocalAtendFactory;
    protected $environment;
    protected $parceiroNegocioService;
    protected $enderecoParceiroService;
    protected $parametrizacaoServicesFactory;
    protected $integracoesTeknisaService;

    public function __construct(EntityManager $entityManager, LocalAtendFactory $LocalAtendFactory, environment $environment, NewCode $newCode, ParceiroNegocioService $parceiroNegocioService, EnderecoParceiroService $enderecoParceiroService, Sm00005000ParametrizacoesServicesFactory $parametrizacaoServicesFactory, IntegracoesTeknisaService $integracoesTeknisaService) {
        $this->entityManager                 = $entityManager;
        $this->connection                    = $entityManager->getConnection();
        $this->LocalAtendFactory             = $LocalAtendFactory;
        $this->environment                   = $environment;
        $this->newCode                       = $newCode;
        $this->parceiroNegocioService        = $parceiroNegocioService;
        $this->enderecoParceiroService       = $enderecoParceiroService;
        $this->parametrizacaoServicesFactory = $parametrizacaoServicesFactory;
        $this->integracoesTeknisaService     = $integracoesTeknisaService;
    }
    
   /**
     * A função "verificaTipoHierarqOrg" Verifica se existe o tipo de Hierarquia Padrão.
     */
     
     public function verificaTipoHierarqOrg($nrorg){
    
        $params = array(
            "NRORG"  => $nrorg
        );
        
        $tpHierOrg = $this->LocalAtendFactory->verificaTipoHierarqOrg($params);
        
        return $tpHierOrg;
    }
    
    
     /**
     * Insere as tabelas ParcNegocio, EndercoParc, EstruturaM, EstruturaH e HierarquiaOrg
     */
    public function inserirTudo($row) {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->environment->getNrOrg();
            $row["NRINSCRICAOPARC"] = preg_replace('/[^0-9]/', '', $row["NRINSCRICAOPARC"]);

            if (isset($row["NRCEPENDERECO"])) {
                $row["NRCEPENDERECO"] = str_replace(array(".", "-"), "", $row["NRCEPENDERECO"]);
            }
            
            /* Tipo de Hierarquia */
            $tpHierOrg = $this->verificaTipoHierarqOrg($nrorg);
            
            if(count($tpHierOrg) > 0){ /* Se ja tiver passa o valor para a  $tipoHierarqOrg */
                $tipoHierarqOrg = $tpHierOrg[0]["NRTIPOHIERARQORG"];
            }
            else{ /* Se não tiver cria e passa o valor para a  $tipoHierarqOrg */
                $tipoHierarqOrg = $this->inserirTipoHierarquiaOrg($row);
            }
           
            /* Parceiro de Negócio */ 
            $parcnegocio = $this->inserirParcNegocio($row);
            
            //Parceria de Estrutura
            $this->inserirParceria($parcnegocio, "ESTRUTURA");
            
            /* Endereço do Parceiro de Negócio */
            $nrenderecoparc = $this->AtualizarEnderecos($row, $parcnegocio);
            
            /* Estrutura M */
            $estruturam = $this->inserirEstruturaM($row, $parcnegocio);
            
            /* Estrutura H */
            $this->inserirEstruturaH($row, $parcnegocio, $estruturam); 
            
            /* Hierarquia */
            $this->inserirHierarquiaOrg($row, $estruturam, $tipoHierarqOrg);

            if (isset($row["IDCONVERTIDOCLIENTE"]) && $row["IDCONVERTIDOCLIENTE"] == "S") {
                if ($row['VINCULA_CLIENTE_EXISTENTE'] == 'N') { 
                    $paramsReplicaParceiro = array(
                        "TIPODEPARCEIRO"  => array("ESTRUTURA", "CLIENTE"),
                        "CDTIPOINSCRICAO" => $row["CDTIPOINSCRICAO"],
                        "NRINSCRICAOPARC" => $row["NRINSCRICAOPARC"],
                        "NMPRINCIPALPARC" => $row["NMESTRUTURAM"],
                        "NMSECUNDARIPARC" => $row["NMESTRUTURAM"],
                        "NRPARCNEGOCIO"   => $parcnegocio,
                        "IDATIVO"         => "S"
                    );
                    
                    $this->parceiroNegocioService->replicaParceiro($paramsReplicaParceiro);
                    
                    if (!empty($nrenderecoparc)) {
                        $enderecoparcRepository = $this->entityManager->getRepository(Enderecoparc::class);
                        
                        $enderecoparc = $enderecoparcRepository->findOneBy(array(
                            "nrenderecoparc" => $nrenderecoparc,
                            "nrorg" => $nrorg
                        ));
                        
                        $paramsReplicaParceiro = array(
                            "TIPODEPARCEIRO" => array("ESTRUTURA", "CLIENTE")
                        );
                        $this->enderecoParceiroService->replicaEnderecoParceiro($enderecoparc, $paramsReplicaParceiro);
                    }
                }
                else  if ($row['VINCULA_CLIENTE_EXISTENTE'] == 'S') {
                    $this->vinculaClienteExistente($row["NRINSCRICAOPARC"], $estruturam, $parcnegocio ,$nrorg);
                }
            }

            /* Chamar APi do ERP para criar filial */
            $chamouIntegracaoERP = null;
            if (isset($row["IDCONVERTIDOFILIAL"]) && $row["IDCONVERTIDOFILIAL"] == "S") {
                $this->verificarFilialRepetidaOutroLocal($row["CDFILIAL"], $estruturam);

                $chamouIntegracaoERP = $this->converterLocalEmFilial($row);
                if ($chamouIntegracaoERP) {
                    $this->atualizarFilialLocalAtendimento($row["CDFILIAL"], $estruturam);
                }
            }
            
            $this->connection->commit();
            
            return [
                "NRESTRUTURAM"        => $estruturam,
                "chamouIntegracaoERP" => $chamouIntegracaoERP,
                "parcnegocio"         => $parcnegocio,
            ];
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    /**
     * Edita as tabelas ParcNegocio, EndercoParc, EstruturaM, EstruturaH e HierarquiaOrg
     */
    public function editarTudo($row) {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->environment->getNrOrg();
            $row["NRINSCRICAOPARC"] = preg_replace('/[^0-9]/', '', $row["NRINSCRICAOPARC"]);

            if (isset($row["NRCEPENDERECO"])) {
                $row["NRCEPENDERECO"] = str_replace(array(".", "-"), "", $row["NRCEPENDERECO"]);
            }
            
            /* Parceiro de Negócio */ 
            $this->editarParcNegocio($row);
            
            /* Endereço do Parceiro de Negócio */

            $this->AtualizarEnderecos($row, $row["NRPARCNEGOCIO"]);
            
            $paramsReplicaParceiro = array(
                "TIPODEPARCEIRO"  => array("ESTRUTURA", "CLIENTE"),
                "CDTIPOINSCRICAO" => $row["CDTIPOINSCRICAO"],
                "NRINSCRICAOPARC" => $row["NRINSCRICAOPARC"],
                "NMPRINCIPALPARC" => $row["NMESTRUTURAM"],
                "NMSECUNDARIPARC" => $row["NMESTRUTURAM"],
                "NRPARCNEGOCIO"   => $row["NRPARCNEGOCIO"],
                "IDATIVO"         => "S"
            );
                    
            if (isset($row["IDCONVERTIDOCLIENTE"]) && $row["IDCONVERTIDOCLIENTE"] == "S") {

                if ($row['VINCULA_CLIENTE_EXISTENTE'] == 'N') {           
                    $this->parceiroNegocioService->replicaParceiro($paramsReplicaParceiro);
                    
                    $nrenderecoparc = $row["NRENDERECOPARC"];
                
                    if (!empty($nrenderecoparc)) {
                        $enderecoparcRepository = $this->entityManager->getRepository(Enderecoparc::class);
                        
                        $enderecoparc = $enderecoparcRepository->findOneBy(array(
                            "nrenderecoparc" => $nrenderecoparc,
                            "nrorg" => $nrorg
                        ));
                        
                        $paramsReplicaParceiro = array(
                            "TIPODEPARCEIRO" => array("ESTRUTURA", "CLIENTE")
                        );
                        $this->enderecoParceiroService->replicaEnderecoParceiro($enderecoparc, $paramsReplicaParceiro);
                    }
                }
                else if ($row['VINCULA_CLIENTE_EXISTENTE'] == 'S') {
                    $this->vinculaClienteExistente($row["NRINSCRICAOPARC"], $row["NRESTRUTURAM"], $row["NRPARCNEGOCIO"],$nrorg);
                }
            }
            else {
                $paramsReplicaParceiro["TIPODEPARCEIRO"] = array("ESTRUTURA");
                $this->parceiroNegocioService->replicaParceiro($paramsReplicaParceiro);
            }

            /* Estrutura M */
            $this->editarEstruturaM($row);
            
            /* Estrutura H */
            $this->editarEstruturaH($row); 
            
            /* Hierarquia */
            $this->editarHierarquiaOrg($row);

            /* Chamar APi do ERP para criar filial */
            $chamouIntegracaoERP = null;
            if (isset($row["IDCONVERTIDOFILIAL"]) && $row["IDCONVERTIDOFILIAL"] == "S") {
                $this->verificarFilialRepetidaOutroLocal($row["CDFILIAL"], $row["NRESTRUTURAM"]);

                $chamouIntegracaoERP = $this->converterLocalEmFilial($row);
                if ($chamouIntegracaoERP) {
                    $this->atualizarFilialLocalAtendimento($row["CDFILIAL"], $row["NRESTRUTURAM"]);
                }
            }
            else {
                //Desvincular filial com o local de atendimento
                $this->atualizarFilialLocalAtendimento(null, $row["NRESTRUTURAM"]);
            }
            
            $this->connection->commit();

            return $chamouIntegracaoERP;
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    /**
     * Exclui as tabelas HierarquiaOrg, EstruturaH, EstruturaM, EndercoParc e ParcNegocio
     */
    public function deletarTudo($row) {
        $nrestruturam = $row['NRESTRUTURAM'];
        $nrorg = $this->environment->getNrOrg();
         
        $params = array(
            "NRESTRUTURAM" => $nrestruturam,
            "NRORG"        => $nrorg
        );
        
        $filhos = $this->LocalAtendFactory->recebefilhos($params);
        
        $this->connection->beginTransaction();

        try {
            
            foreach($filhos as $filho){
                
                $estruturam = $filho['NRESTRUTURAM'];
                $parcnegocio = $filho['NRPARCNEGOCIO'];
                
                /* Hierarquia */
                $this->deletarHierarquiaOrg($estruturam);
                
                /* Estrutura H */
                $this->deletarEstruturaH($estruturam); 
                
                /* Estrutura M */
                $this->deletarEstruturaM($estruturam);
                
                /* Endereço do Parceiro de Negócio */
                $this->deletarEnderecoParc($parcnegocio);
                
                /* Parceiro de Negócio */ 
                $this->deletarParcNegocio($parcnegocio);
            }

            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    /**
     * Salva o Tipo de Hierarquia.
     */
    public function inserirTipoHierarquiaOrg($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create();
        
        $novoCodigo = $this->newCode->getCodeByTableName("TIPOHIERARQORG", $nrorg, 1, 10, true);
        
        $params = array(
            "NRTIPOHIERARQORG"  => $novoCodigo,
            "NMTIPOHIERARQORG"  => 'Padrão - Local de Atendimento SM',
            "NRTIPOESTRUTSUP"   => $row["NRTIPOESTRUTURA"],
            "NRTIPOESTRUTREL"   => $row["NRTIPOESTRUTURA"],
            "NRORG"             => $nrorg,
            "DTINCLUSAO"        => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"     => $nrorg,
            "CDOPERINCLUSAO"    => $cdoperador,
            "IDATIVO"           => "S"
        );
        
        $this->LocalAtendFactory->inserirTipoHierarquiaOrg($params);
        
        return $novoCodigo;
    }
    
     /**
     * Salva o Parceiro de Negócio.
     */
    public function inserirParcNegocio($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create(); 
        
        $cdtipoparcprincipal = "ESTRUTURA";
        if (isset($row["IDCONVERTIDOCLIENTE"]) && $row["IDCONVERTIDOCLIENTE"] == "S")
            $cdtipoparcprincipal = "CLIENTE";
        
        $novoCodigo = $this->newCode->getCodeByTableName("PARCNEGOCIO", $nrorg, 1, 10, true);
        
        $params = array(
            "NRPARCNEGOCIO"       => $novoCodigo,
            "NMPRINCIPALPARC"     => $row["NMESTRUTURAM"],
            "NMSECUNDARIPARC"     => $row["NMESTRUTURAM"],
            "NRINSCRICAOPARC"     => $row["NRINSCRICAOPARC"],
            "CDTIPOPARCPRINCIPAL" => $cdtipoparcprincipal,
            "CDTIPOINSCRICAO"     => $row["CDTIPOINSCRICAO"],
            "IDPESSOAFISICA"      => "N",
            "IDINSTITUICAO"       => "S",
            "IDPARCFUNDIDO"       => "N",
            "NRORG"               => $nrorg,
            "DTINCLUSAO"          => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"       => $nrorg,
            "CDOPERINCLUSAO"      => $cdoperador,
            "IDATIVO"             => "S"
        );
        
        $this->LocalAtendFactory->inserirParcNegocio($params);
        
        return $novoCodigo;
    }
    
    public function inserirParceria($parcnegocio, $cdtipoparceiro) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $nrparceria = $this->newCode->getCodeByTableName("PARCERIA", $nrorg, 1, 10, true);
        $params = array(
            "NRPARCERIA"     => $nrparceria,
            "NRPARCNEGOCIO"  => $parcnegocio,
            "CDTIPOPARCEIRO" => $cdtipoparceiro,
            "NRORG"          => $nrorg,
            "IDATIVO"        => "S",
            "NRORGINCLUSAO"  => $nrorg,
            "CDOPERINCLUSAO" => $cdoperador
        );
        
        $this->LocalAtendFactory->inserirParceria($params);
    }
    
    /**
     * atualiza o Parceiro de Negócio.
     */
    public function editarParcNegocio($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $cdtipoparcprincipal = "ESTRUTURA";
        if (isset($row["IDCONVERTIDOCLIENTE"]) && $row["IDCONVERTIDOCLIENTE"] == "S") {
            $cdtipoparcprincipal = "CLIENTE";
        }
        
        $params = array(
            "NMPRINCIPALPARC"     => $row["NMESTRUTURAM"],
            "NMSECUNDARIPARC"     => $row["NMESTRUTURAM"],
            "NRINSCRICAOPARC"     => $row["NRINSCRICAOPARC"],
            "CDTIPOINSCRICAO"     => $row["CDTIPOINSCRICAO"],
            "CDTIPOPARCPRINCIPAL" => $cdtipoparcprincipal,
            "CDOPERULTATU"        => $cdoperador,
            "NRORG"               => $nrorg,
            "NRPARCNEGOCIO"       => $row["NRPARCNEGOCIO"]
        );
        
        $this->LocalAtendFactory->editarParcNegocio($params);
    }
    
     /**
     * Deleta o Parceiro de Negócio.
     */
    public function deletarParcNegocio($nrparncnegocio) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRPARCNEGOCIO"       => $nrparncnegocio,
            "NRORG"               => $nrorg
        );
        
        $this->LocalAtendFactory->deletarParcNegocio($params);
    }

     /**
     * Salva a Estrutura M.
     */
    public function inserirEstruturaM($row, $parcnegocio) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create();
            
        $novoCodigo = $this->newCode->getCodeByTableName("ESTRUTURAM", $nrorg, 1, 10, true);
        
        $params = array(
            "NRESTRUTURAM"     => $novoCodigo,
            "NMESTRUTURAM"     => $row["NMESTRUTURAM"],
            "NRTIPOESTRUTURA"  => $row["NRTIPOESTRUTURA"],
            "DTINIVIGENCIA"    => $dataAtual->format('d/m/Y'),         
            "NRPARCNEGOCIO"    => $parcnegocio,
            "CDCENTCUST"       => $row["CDCENTCUST"],
            //"DSEMAILESTRUTURA" => $row["DSEMAILESTRUTURA"],
            "NMRAZSOCFILI"     => $row['NMRAZSOCFILI'],
            "NRORG"            => $nrorg,
            "DTINCLUSAO"       => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"    => $nrorg,
            "CDOPERINCLUSAO"   => $cdoperador,
            "IDATIVO"          => "S"
        );
        
        $this->LocalAtendFactory->inserirEstruturaM($params);
        
        return $novoCodigo;
    }    
    
     /**
     * Atualiza a Estrutura M.
     */
    public function editarEstruturaM($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $params = array(
            "NMESTRUTURAM"     => $row["NMESTRUTURAM"],
            "NRTIPOESTRUTURA"  => $row["NRTIPOESTRUTURA"],
            "CDCENTCUST"       => $row["CDCENTCUST"],
            "DSEMAILESTRUTURA" => $row["DSEMAILESTRUTURA"],
            "NMRAZSOCFILI"     => $row['NMRAZSOCFILI'],
            "CDOPERULTATU"     => $cdoperador,
            "NRORG"            => $nrorg,
            "NRESTRUTURAM"     => $row["NRESTRUTURAM"]
        );
        
        $this->LocalAtendFactory->editarEstruturaM($params);
    }   
    
    /**
     * Deleta a Estrutura M.
     */
    public function deletarEstruturaM($nrestruturam) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRESTRUTURAM"    => $nrestruturam,
            "NRORG"           => $nrorg
        );
        
        $this->LocalAtendFactory->deletarEstruturaM($params);
    }   
    
    /**
     * Salva a Estrutura H.
     */
    public function inserirEstruturaH($row, $parcnegocio, $estruturam) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create();
        
        $novoCodigo = $this->newCode->getCodeByTableName("ESTRUTURAH", $nrorg, 1, 10, true);
        
        $params = array(
            "NRESTRUTURAH"   => $novoCodigo,
            "NMESTRUTURAH"   => $row["NMESTRUTURAM"],
            "NRESTRUTURAM"   => $estruturam,
            "DTMESCOMPETENC" => $dataAtual->format('d/m/Y'),
            "NRPARCNEGOCIO"  => $parcnegocio,
            "NRORG"          => $nrorg,
            "DTINCLUSAO"     => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"  => $nrorg,
            "CDOPERINCLUSAO" => $cdoperador,
            "IDATIVO"        => "S"
        );
        
        $this->LocalAtendFactory->inserirEstruturaH($params);
    }
    
    /**
     * Atualiza a Estrutura H.
     */
    public function editarEstruturaH($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $params = array(
            "NMESTRUTURAM"   => $row["NMESTRUTURAM"],
            "CDOPERULTATU"   => $cdoperador,
            "NRORG"          => $nrorg,
            "NRESTRUTURAH"   => $row["NRESTRUTURAH"],
            "NRESTRUTURAM"   => $row["NRESTRUTURAM"]
        );
        
        $this->LocalAtendFactory->editarEstruturaH($params);
    }
    
    /**
     * Deleta a Estrutura H.
     */
    public function deletarEstruturaH($nrestruturam) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRESTRUTURAM"    => $nrestruturam,
            "NRORG"           => $nrorg
        );
        
        $this->LocalAtendFactory->deletarEstruturaH($params);
    }
    
     /**
     * Salva a Hierarquia da Estrutura.
     */
    public function inserirHierarquiaOrg($row, $estruturam, $tipoHierarqOrg) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create();
        
        $novoCodigo = $this->newCode->getCodeByTableName("HIERARQUIAORG", $nrorg, 1, 10, true);
        
        $params = array(
            "NRHIERARQUIAORG"  => $novoCodigo,
            "NRTIPOHIERARQORG" => $tipoHierarqOrg,
            "NRESTRUTURAMSUP"  => $row["NRESTRUTURAMSUP"], 
            "NRESTRUTURAMREL"  => $estruturam,
            "DTINIVIGENCIA"    => $dataAtual->format('d/m/Y'),
            "NRORG"            => $nrorg,
            "DTINCLUSAO"       => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"    => $nrorg,
            "CDOPERINCLUSAO"   => $cdoperador,
            "IDATIVO"          => "S"
        );
        
        $this->LocalAtendFactory->inserirHierarquiaOrg($params);
    }
    
     /**
     * atualiza a Hierarquia da Estrutura.
     */
    public function editarHierarquiaOrg($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $params = array(
            "NRESTRUTURAMSUP"  => $row["NRESTRUTURAMSUP"], 
            "NRESTRUTURAMREL"  => $row["NRESTRUTURAM"], 
            "CDOPERULTATU"     => $cdoperador,
            "NRORG"            => $nrorg,
            "NRHIERARQUIAORG"  => $row["NRHIERARQUIAORG"]
        );
        
        $this->LocalAtendFactory->editarHierarquiaOrg($params);
    }
    
     /**
     * Deleta a Hierarquia da Estrutura.
     */
    public function deletarHierarquiaOrg($nrestruturam) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRESTRUTURAM"    => $nrestruturam,
            "NRORG"           => $nrorg
        );
        
        $this->LocalAtendFactory->deletarHierarquiaOrg($params);
    }
    
    /**
     * Salva o endereço de Hierarquia da Estrutura.
     */
    public function AtualizarEnderecos($row, $parcnegocio) {

         $nrorg = $this->environment->getNrOrg();
         $cdoperador = $this->environment->getCdOperador();
         $dataAtual = date_create();
         $novoCodigoPrincipal = null;
        
        $params = array(
            "NRPARCNEGOCIO"    => $parcnegocio, 
            "CDTIPOENDERECO"   => "PRINCIPAL",
            "NRORG"            => $nrorg
        );
        $dataset = $this->LocalAtendFactory->getEnderecoParceiro($params);
        if (count($dataset) > 0) {

            $principal = array(
                "DSENDERECO"       => $row["DSENDERECO"],
                "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO"],
                "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO"],
                "CDPAIS"           => $row["CDPAIS"],
                "SGESTADO"         => $row["SGESTADO"],
                "CDMUNICIPIO"      => $row["CDMUNICIPIO"],
                "CDBAIRRO"         => $row["CDBAIRRO"],
                "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO"],
                "NRCEPENDERECO"    => $row["NRCEPENDERECO"],
                "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE"],
                "CDOPERULTATU"     => $cdoperador,
                "NRORG"            => $nrorg,
                "CDTIPOENDERECO"   => "PRINCIPAL",
                "NRPARCNEGOCIO"    => $row["NRPARCNEGOCIO"]
            );
           
            $this->LocalAtendFactory->editarEnderecoParc($principal);
        }
        else {
            $novoCodigoPrincipal = $this->newCode->getCodeByTableName("ENDERECOPARC", $nrorg, 1, 10, true);
        
            $principal = array(
                "NRENDERECOPARC"   => $novoCodigoPrincipal,
                "NRPARCNEGOCIO"    => $parcnegocio, 
                "CDTIPOENDERECO"   => "PRINCIPAL",
                "DSENDERECO"       => $row["DSENDERECO"],
                "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO"],
                "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO"],
                "CDPAIS"           => $row["CDPAIS"],
                "SGESTADO"         => $row["SGESTADO"],
                "CDMUNICIPIO"      => $row["CDMUNICIPIO"],
                "CDBAIRRO"         => $row["CDBAIRRO"],
                "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO"],
                "NRCEPENDERECO"    => $row["NRCEPENDERECO"],
                "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE"],
                "NRORG"            => $nrorg,
                "DTINCLUSAO"       => $dataAtual->format('d/m/Y'),
                "NRORGINCLUSAO"    => $nrorg,
                "CDOPERINCLUSAO"   => $cdoperador,
                "IDATIVO"          => "S"
            );
           
            $this->LocalAtendFactory->inserirEnderecoParc($principal);
        }

        

         if((isset($row["NRCEPENDERECO_EXTERIOR"])) && ($row["NRCEPENDERECO_EXTERIOR"] !== '')){
           
            $params = array(
                "NRPARCNEGOCIO"    => $parcnegocio, 
                "CDTIPOENDERECO"   => "EXTERIOR",
                "NRORG"            => $nrorg
            );
            $dataset = $this->LocalAtendFactory->getEnderecoParceiro($params);
            if (count($dataset) > 0) {
    
                $exterior = array(
                    "DSENDERECO"       => $row["DSENDERECO_EXTERIOR"],
                    "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO_EXTERIOR"],
                    "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO_EXTERIOR"],
                    "CDPAIS"           => $row["CDPAIS_EXTERIOR"],
                    "SGESTADO"         => $row["SGESTADO_EXTERIOR"],
                    "CDMUNICIPIO"      => $row["CDMUNICIPIO_EXTERIOR"],
                    "CDBAIRRO"         => $row["CDBAIRRO_EXTERIOR"],
                    "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO_EXTERIOR"],
                    "NRCEPENDERECO"    => $row["NRCEPENDERECO_EXTERIOR"],
                    "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE_EXTERIOR"],
                    "CDOPERULTATU"     => $cdoperador,
                    "NRORG"            => $nrorg,
                    "CDTIPOENDERECO"   => "EXTERIOR",
                    "NRPARCNEGOCIO"    => $row["NRPARCNEGOCIO"]
                );
                    
                $this->LocalAtendFactory->editarEnderecoParc($exterior);
            }
            else {
                $novoCodigoExterior = $this->newCode->getCodeByTableName("ENDERECOPARC", $nrorg, 1, 10, true);
                $exterior = array(
                    "NRENDERECOPARC"   => $novoCodigoExterior,
                    "NRPARCNEGOCIO"    => $parcnegocio, 
                    "CDTIPOENDERECO"   => "EXTERIOR",
                    "DSENDERECO"       => $row["DSENDERECO_EXTERIOR"],
                    "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO_EXTERIOR"],
                    "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO_EXTERIOR"],
                    "CDPAIS"           => $row["CDPAIS_EXTERIOR"],
                    "SGESTADO"         => $row["SGESTADO_EXTERIOR"],
                    "CDMUNICIPIO"      => $row["CDMUNICIPIO_EXTERIOR"],
                    "CDBAIRRO"         => $row["CDBAIRRO_EXTERIOR"],
                    "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO_EXTERIOR"],
                    "NRCEPENDERECO"    => $row["NRCEPENDERECO_EXTERIOR"],
                    "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE_EXTERIOR"],
                    "NRORG"            => $nrorg,
                    "DTINCLUSAO"       => $dataAtual->format('d/m/Y'),
                    "NRORGINCLUSAO"    => $nrorg,
                    "CDOPERINCLUSAO"   => $cdoperador,
                    "IDATIVO"          => "S"
                );
                    
                $this->LocalAtendFactory->inserirEnderecoParc($exterior);
            }
           
        }  
        
        if((isset($row["NRCEPENDERECO_COBRANCA"])) && ($row["NRCEPENDERECO_COBRANCA"] !== '')){
            
            $params = array(
                "NRPARCNEGOCIO"    => $parcnegocio, 
                "CDTIPOENDERECO"   => "COBRANCA",
                "NRORG"            => $nrorg
            );
            $dataset = $this->LocalAtendFactory->getEnderecoParceiro($params);
            if (count($dataset) > 0) {
    
                $cobranca = array(
                    "DSENDERECO"       => $row["DSENDERECO_COBRANCA"],
                    "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO_COBRANCA"],
                    "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO_COBRANCA"],
                    "CDPAIS"           => $row["CDPAIS_COBRANCA"],
                    "SGESTADO"         => $row["SGESTADO_COBRANCA"],
                    "CDMUNICIPIO"      => $row["CDMUNICIPIO_COBRANCA"],
                    "CDBAIRRO"         => $row["CDBAIRRO_COBRANCA"],
                    "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO_COBRANCA"],
                    "NRCEPENDERECO"    => $row["NRCEPENDERECO_COBRANCA"],
                    "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE_COBRANCA"],
                    "CDOPERULTATU"     => $cdoperador,
                    "NRORG"            => $nrorg,
                    "CDTIPOENDERECO"   => "COBRANCA",
                    "NRPARCNEGOCIO"    => $row["NRPARCNEGOCIO"]
                );
                    
                $this->LocalAtendFactory->editarEnderecoParc($cobranca);
            }
            else {
                $novoCodigoCobranca = $this->newCode->getCodeByTableName("ENDERECOPARC", $nrorg, 1, 10, true);
                $cobranca = array(
                    "NRENDERECOPARC"   => $novoCodigoCobranca,
                    "NRPARCNEGOCIO"    => $parcnegocio, 
                    "CDTIPOENDERECO"   => "COBRANCA",
                    "DSENDERECO"       => $row["DSENDERECO_COBRANCA"],
                    "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO_COBRANCA"],
                    "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO_COBRANCA"],
                    "CDPAIS"           => $row["CDPAIS_COBRANCA"],
                    "SGESTADO"         => $row["SGESTADO_COBRANCA"],
                    "CDMUNICIPIO"      => $row["CDMUNICIPIO_COBRANCA"],
                    "CDBAIRRO"         => $row["CDBAIRRO_COBRANCA"],
                    "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO_COBRANCA"],
                    "NRCEPENDERECO"    => $row["NRCEPENDERECO_COBRANCA"],
                    "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE_COBRANCA"],
                    "NRORG"            => $nrorg,
                    "DTINCLUSAO"       => $dataAtual->format('d/m/Y'),
                    "NRORGINCLUSAO"    => $nrorg,
                    "CDOPERINCLUSAO"   => $cdoperador,
                    "IDATIVO"          => "S"
                );
                    
                $this->LocalAtendFactory->inserirEnderecoParc($cobranca);
            }
           
        }  

        if((isset($row["NRCEPENDERECO_ENTREGA"])) && ($row["NRCEPENDERECO_ENTREGA"] !== '')){
           
            $params = array(
                "NRPARCNEGOCIO"    => $parcnegocio, 
                "CDTIPOENDERECO"   => "ENTREGA",
                "NRORG"            => $nrorg 
            );
            $dataset = $this->LocalAtendFactory->getEnderecoParceiro($params);
            if (count($dataset) > 0) {
    
                $entrega = array(
                    "DSENDERECO"       => $row["DSENDERECO_ENTREGA"],
                    "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO_ENTREGA"],
                    "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO_ENTREGA"],
                    "CDPAIS"           => $row["CDPAIS_ENTREGA"],
                    "SGESTADO"         => $row["SGESTADO_ENTREGA"],
                    "CDMUNICIPIO"      => $row["CDMUNICIPIO_ENTREGA"],
                    "CDBAIRRO"         => $row["CDBAIRRO_ENTREGA"],
                    "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO_ENTREGA"],
                    "NRCEPENDERECO"    => $row["NRCEPENDERECO_ENTREGA"],
                    "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE_ENTREGA"],
                    "CDOPERULTATU"     => $cdoperador,
                    "NRORG"            => $nrorg,
                    "CDTIPOENDERECO"   => "ENTREGA",
                    "NRPARCNEGOCIO"    => $row["NRPARCNEGOCIO"]
                );
                    
                $this->LocalAtendFactory->editarEnderecoParc($entrega);
            }
            else {
                $novoCodigoEntrega = $this->newCode->getCodeByTableName("ENDERECOPARC", $nrorg, 1, 10, true);
                $entrega = array(
                    "NRENDERECOPARC"   => $novoCodigoEntrega,
                    "NRPARCNEGOCIO"    => $parcnegocio, 
                    "CDTIPOENDERECO"   => "ENTREGA",
                    "DSENDERECO"       => $row["DSENDERECO_ENTREGA"],
                    "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO_ENTREGA"],
                    "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO_ENTREGA"],
                    "CDPAIS"           => $row["CDPAIS_ENTREGA"],
                    "SGESTADO"         => $row["SGESTADO_ENTREGA"],
                    "CDMUNICIPIO"      => $row["CDMUNICIPIO_ENTREGA"],
                    "CDBAIRRO"         => $row["CDBAIRRO_ENTREGA"],
                    "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO_ENTREGA"],
                    "NRCEPENDERECO"    => $row["NRCEPENDERECO_ENTREGA"],
                    "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE_ENTREGA"],
                    "NRORG"            => $nrorg,
                    "DTINCLUSAO"       => $dataAtual->format('d/m/Y'),
                    "NRORGINCLUSAO"    => $nrorg,
                    "CDOPERINCLUSAO"   => $cdoperador,
                    "IDATIVO"          => "S"
                );
                    
                $this->LocalAtendFactory->inserirEnderecoParc($entrega);
            }

            
        }
        
        return $novoCodigoPrincipal;
    }
    
    /**
     * atualiza o endereço de Hierarquia da Estrutura.
     */
    public function editarEnderecoParc($row) {
         $nrorg = $this->environment->getNrOrg();
         $cdoperador = $this->environment->getCdOperador();

         $principal = array(
             "DSENDERECO"       => $row["DSENDERECO"],
             "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO"],
             "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO"],
             "CDPAIS"           => $row["CDPAIS"],
             "SGESTADO"         => $row["SGESTADO"],
             "CDMUNICIPIO"      => $row["CDMUNICIPIO"],
             "CDBAIRRO"         => $row["CDBAIRRO"],
             "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO"],
             "NRCEPENDERECO"    => $row["NRCEPENDERECO"],
             "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE"],
             "CDOPERULTATU"     => $cdoperador,
             "NRORG"            => $nrorg,
             "CDTIPOENDERECO"   => "PRINCIPAL",
             "NRPARCNEGOCIO"    => $row["NRPARCNEGOCIO"]
         );
        
         $this->LocalAtendFactory->editarEnderecoParc($principal);

         if((isset($row["NRCEPENDERECO_EXTERIOR"])) && ($row["NRCEPENDERECO_EXTERIOR"] !== '')){
           $exterior = array(
                "DSENDERECO"       => $row["DSENDERECO_EXTERIOR"],
                "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO_EXTERIOR"],
                "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO_EXTERIOR"],
                "CDPAIS"           => $row["CDPAIS_EXTERIOR"],
                "SGESTADO"         => $row["SGESTADO_EXTERIOR"],
                "CDMUNICIPIO"      => $row["CDMUNICIPIO_EXTERIOR"],
                "CDBAIRRO"         => $row["CDBAIRRO_EXTERIOR"],
                "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO_EXTERIOR"],
                "NRCEPENDERECO"    => $row["NRCEPENDERECO_EXTERIOR"],
                "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE_EXTERIOR"],
                "CDOPERULTATU"     => $cdoperador,
                "NRORG"            => $nrorg,
                "CDTIPOENDERECO"   => "EXTERIOR",
                "NRPARCNEGOCIO"    => $row["NRPARCNEGOCIO"]
            );
                
            $this->LocalAtendFactory->editarEnderecoParc($exterior);
        }  
        
        if((isset($row["NRCEPENDERECO_COBRANCA"])) && ($row["NRCEPENDERECO_COBRANCA"] !== '')){
            
            $cobranca = array(
                "DSENDERECO"       => $row["DSENDERECO_COBRANCA"],
                "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO_COBRANCA"],
                "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO_COBRANCA"],
                "CDPAIS"           => $row["CDPAIS_COBRANCA"],
                "SGESTADO"         => $row["SGESTADO_COBRANCA"],
                "CDMUNICIPIO"      => $row["CDMUNICIPIO_COBRANCA"],
                "CDBAIRRO"         => $row["CDBAIRRO_COBRANCA"],
                "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO_COBRANCA"],
                "NRCEPENDERECO"    => $row["NRCEPENDERECO_COBRANCA"],
                "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE_COBRANCA"],
                "CDOPERULTATU"     => $cdoperador,
                "NRORG"            => $nrorg,
                "CDTIPOENDERECO"   => "COBRANCA",
                "NRPARCNEGOCIO"    => $row["NRPARCNEGOCIO"]
            );
                
            $this->LocalAtendFactory->editarEnderecoParc($cobranca);
        } 
        
        if((isset($row["NRCEPENDERECO_ENTREGA"])) && ($row["NRCEPENDERECO_ENTREGA"] !== '')){
           
            $entrega = array(
                "DSENDERECO"       => $row["DSENDERECO_ENTREGA"],
                "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO_ENTREGA"],
                "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO_ENTREGA"],
                "CDPAIS"           => $row["CDPAIS_ENTREGA"],
                "SGESTADO"         => $row["SGESTADO_ENTREGA"],
                "CDMUNICIPIO"      => $row["CDMUNICIPIO_ENTREGA"],
                "CDBAIRRO"         => $row["CDBAIRRO_ENTREGA"],
                "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO_ENTREGA"],
                "NRCEPENDERECO"    => $row["NRCEPENDERECO_ENTREGA"],
                "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE_ENTREGA"],
                "CDOPERULTATU"     => $cdoperador,
                "NRORG"            => $nrorg,
                "CDTIPOENDERECO"   => "ENTREGA",
                "NRPARCNEGOCIO"    => $row["NRPARCNEGOCIO"]
            );
                
            $this->LocalAtendFactory->editarEnderecoParc($entrega);
        }
    }
    
     /**
     * Deleta o endereço de Hierarquia da Estrutura.
     */
    public function deletarEnderecoParc($nrparncnegocio) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRPARCNEGOCIO"       => $nrparncnegocio,
            "NRORG"               => $nrorg
        );
        
        $this->LocalAtendFactory->deletarEnderecoParc($params);
    }
    
    public function verificarExisteClienteComMesmaInscricao($nrparcnegocio, $nrinscricaoparc, $nrorg) {
        $params = array(
            "NRINSCRICAOPARC" => $nrinscricaoparc,
            "NRORG"           => $nrorg,
            "NRPARCNEGOCIO"   => $nrparcnegocio
        );
        
        $dataset = $this->LocalAtendFactory->verificarExisteClienteComMesmaInscricao($params);
        return ((int)$dataset[0]["QTD"] > 0);
    }

    public function verificarExisteClienteComMesmaParceria($nrinscricaoparc) {
        $nrorg = $this->environment->getNrOrg();
        $retorno = 'S';

        $params = array(
            "NRINSCRICAOPARC" => $nrinscricaoparc,
            "NRORG"           => $nrorg
        );
        
        $dataset = $this->LocalAtendFactory->verificarExisteClienteComMesmaParceria($params);

        if (count($dataset) > 0) {
            $retorno = 'PERGUNTA';
        }
        else {
            $retorno = 'S';     
        }

        return $retorno;
    }

    public function vinculaClienteExistente($nrinscricaoparc, $estruturam, $parcAnte, $nrorg){
        $cdoperador = $this->environment->getCdOperador();

        /** 1 - Criar a parceria de estrutura, caso não exista, para o parceiro de negócio do tipo cliente */
        $params = array(
            "NRINSCRICAOPARC" => $nrinscricaoparc,
            "NRORG"           => $nrorg
        );

        $parccliente = $this->LocalAtendFactory->getParceiroCliente($params);   
        
        $params = array(
            "NRPARCNEGOCIO" => $parccliente,
            "NRORG"           => $nrorg
        );

        $validaEstrut = $this->LocalAtendFactory->validaParceriaEstrutura($params);   
        
        if ($validaEstrut == 0) {

            $novoCodigo= $this->newCode->getCodeByTableName("PARCERIA", $nrorg, 1, 10, true);

            $paramsIns = array(
                "NRPARCERIA"    => $novoCodigo,
                "NRPARCNEGOCIO" => $parccliente,
                "NRORG"         => $nrorg,
                "OPERADOR"      => $cdoperador
            );

            $this->LocalAtendFactory->insereParceriaEstrutura($paramsIns);   

        }
  
        /** 2 - Vincular o registro do Local de Atendimento à este parceiro de negócio */

        $paramsVinc = array(
            "NRPARCNEGOCIO" => $parccliente,
            "NRESTRUTURAM"  => $estruturam,
            "NRORG"         => $nrorg
        );

        $this->LocalAtendFactory->VinculaParceiroAEstrutura($paramsVinc);   

        /** 3 - Inativar o parceiro de negócio anterior */

        $paramsInat = array(
            "NRPARCNEGOCIO" => $parcAnte,
            "NRORG"         => $nrorg
        );

        $this->LocalAtendFactory->InativaParcNegocioAnte($paramsInat);  
    }

    public function converterLocalEmFilial($row) {

        $chamouIntegracao = false;
        $urlIntegracaoErp = $this->getUrlIntegracaoErpFilial();

        $url = $urlIntegracaoErp . "filial_register";

        if (!empty($urlIntegracaoErp)) {
            $isNew = true;
            $filial = $this->getFilial($row["CDFILIAL"]);
            if (!is_null($filial)) {
                $isNew = false;
            }

            $tiposInscricao = [
                "CNPJ"   => "J",
                "CPF"    => "F",
                "CODIGO" => "O",
                "LIVRE"  => "I"
            ];

            $dadosOperador = $this->getDadosOperador();
            $email = $dadosOperador["DSEMAILOPER"];
            $senha = $dadosOperador["CDSENHAOPERWEB"];

            $dadosIntegracao = [
                "__is_new"      => $isNew,
                "changed"       => true,
                "CDFILIAL"      => $row["CDFILIAL"],
                "NMFILIAL"      => substr($row["NMESTRUTURAM"], 0, 30),
                "NMRAZSOCFILI"  => $row["NMRAZSOCFILI"],
                "IDTPIJURFILI"  => $tiposInscricao[$row["CDTIPOINSCRICAO"]],
                "NRINSJURFILI"  => $row["NRINSCRICAOPARC"],
                "CDEMPRESA"     => $row["CDEMPRESA"],
                "NMRAZSOCEMPR"  => $row["NMEMPRESA"],
                "CDINSCESTA"    => $row["CDINSCESTA"],
                "CDINSCMUNI"    => $row["CDINSCMUNI"],
                "IDTPENDEFILI"  => "P",
                "CDPAIS"        => $row["CDPAIS"],
                "SGESTADO"      => $row["SGESTADO"],
                "CDMUNICIPIO"   => $row["CDMUNICIPIO"],
                "DSENDEFILISEP" => $row["DSENDERECO"],
                "NRENDEFILISEP" => $row["NRIMOVELENDERECO"],
                "NMBAIRFILI"    => $row["NMBAIRROENDERECO"],
                "NRCEPFILI"     => $row["NRCEPENDERECO"],
                "DSCOMPENDFIL"  => $row["DSCOMPLEENDERECO"],
                "IDFUNCAOFILI"  => null
            ];

            $chamouIntegracao = true;
            try {
                $this->integracoesTeknisaService->enviarDadosIntegracaoErpFilial($url, $dadosIntegracao, $email, $senha);
                
                $this->enviarOutrosEnderecos($urlIntegracaoErp, $row, $email, $senha, $tiposInscricao);
            }
            catch (\Exception $e) {
                throw new \Exception("Falha na integração com o ERP:<br>" . $e->getMessage());
            }
        }
        
        return $chamouIntegracao;
    }

    public function enviarOutrosEnderecos($urlIntegracaoErp, $row, $email, $senha, $tiposInscricao) 
    {   
        $url = $urlIntegracaoErp . "endeFili_register";
         
        
        $dadosIntegracao = [
            "changed"       => true,
            "CDFILIAL"      => $row["CDFILIAL"],
            "NMFILIAL"      => substr($row["NMESTRUTURAM"], 0, 30),
            "NMRAZSOCFILI"  => $row["NMESTRUTURAM"],
            "IDTPIJURFILI"  => $tiposInscricao[$row["CDTIPOINSCRICAO"]],
            "NRINSJURFILI"  => $row["NRINSCRICAOPARC"],
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "NMRAZSOCEMPR"  => $row["NMEMPRESA"],
            "CDINSCESTA"    => $row["CDINSCESTA"],
            "CDINSCMUNI"    => $row["CDINSCMUNI"],
            "IDFUNCAOFILI"  => null
        ];
         
       // Endereço Principal

       if((isset($row["NRCEPENDERECO"])) && ($row["NRCEPENDERECO"] !== '') && ($row["NRCEPENDERECO"] !== null)){

            $paramsValida = array(
                "CDFILIAL"      => $row["CDFILIAL"],
                "IDTPENDEFILI"  => "P",
            );

            $valida = $this->LocalAtendFactory->verificarExisteEndeFili($paramsValida);

            if ($valida == 0) {
                $isNew = true;
            }
            else {
                $isNew = false;
            };

            $exterior = array(
                "__is_new"      => $isNew,
                "CDPAIS"        => $row["CDPAIS"],
                "SGESTADO"      => $row["SGESTADO"],
                "CDMUNICIPIO"   => $row["CDMUNICIPIO"],
                "DSENDEFILISEP" => $row["DSENDERECO"],
                "NRENDEFILISEP" => $row["NRIMOVELENDERECO"],
                "NMBAIRFILI"    => $row["NMBAIRROENDERECO"],
                "NRCEPFILI"     => $row["NRCEPENDERECO"],
                "DSCOMPENDFIL"  => $row["DSCOMPLEENDERECO"],
                "IDTPENDEFILI"  => "P",
            );
            
            $this->integracoesTeknisaService->enviarDadosIntegracaoErpFilialEnde($url, array_merge($dadosIntegracao, $exterior), $email, $senha);
        }  

        // Endereço Exterior
        if((isset($row["NRCEPENDERECO_EXTERIOR"])) && ($row["NRCEPENDERECO_EXTERIOR"] !== '') && ($row["NRCEPENDERECO_EXTERIOR"] !== null)){

            $paramsValida = array(
                "CDFILIAL"      => $row["CDFILIAL"],
                "IDTPENDEFILI"  => "E",
            );

            $valida = $this->LocalAtendFactory->verificarExisteEndeFili($paramsValida);

            if ($valida == 0) {
                $isNew = true;
            }
            else {
                $isNew = false;
            };

            $exterior = array(
                "__is_new"      => $isNew,
                "CDPAIS"        => $row["CDPAIS_EXTERIOR"],
                "SGESTADO"      => $row["SGESTADO_EXTERIOR"],
                "CDMUNICIPIO"   => $row["CDMUNICIPIO_EXTERIOR"],
                "DSENDEFILISEP" => $row["DSENDERECO_EXTERIOR"],
                "NRENDEFILISEP" => $row["NRIMOVELENDERECO_EXTERIOR"],
                "NMBAIRFILI"    => $row["NMBAIRROENDERECO_EXTERIOR"],
                "NRCEPFILI"     => $row["NRCEPENDERECO_EXTERIOR"],
                "DSCOMPENDFIL"  => $row["DSCOMPLEENDERECO_EXTERIOR"],
                "IDTPENDEFILI"  => "E",
            );
            
            $this->integracoesTeknisaService->enviarDadosIntegracaoErpFilialEnde($url, array_merge($dadosIntegracao, $exterior), $email, $senha);
        }  
        
        // Endereço de Cobrança
        if((isset($row["NRCEPENDERECO_COBRANCA"])) && ($row["NRCEPENDERECO_COBRANCA"] !== '') &&  ($row["NRCEPENDERECO_COBRANCA"] !== null)){
          
            $paramsValida = array(
                "CDFILIAL"      => $row["CDFILIAL"],
                "IDTPENDEFILI"  => "C",
            );

            $valida = $this->LocalAtendFactory->verificarExisteEndeFili($paramsValida);

            if ($valida == 0) {
                $isNew = true;
            }
            else {
                $isNew = false;
            };

            $cobranca = array(
                "__is_new"      => $isNew,
                "CDPAIS"        => $row["CDPAIS_COBRANCA"],
                "SGESTADO"      => $row["SGESTADO_COBRANCA"],
                "CDMUNICIPIO"   => $row["CDMUNICIPIO_COBRANCA"],
                "DSENDEFILISEP" => $row["DSENDERECO_COBRANCA"],
                "NRENDEFILISEP" => $row["NRIMOVELENDERECO_COBRANCA"],
                "NMBAIRFILI"    => $row["NMBAIRROENDERECO_COBRANCA"],
                "NRCEPFILI"     => $row["NRCEPENDERECO_COBRANCA"],
                "DSCOMPENDFIL"  => $row["DSCOMPLEENDERECO_COBRANCA"],
                "IDTPENDEFILI"  => "C",
            );

            $this->integracoesTeknisaService->enviarDadosIntegracaoErpFilialEnde($url, array_merge($dadosIntegracao, $cobranca), $email, $senha);
        } 

        //Endereço de Entrega
        if((isset($row["NRCEPENDERECO_ENTREGA"])) && ($row["NRCEPENDERECO_ENTREGA"] !== '') && ($row["NRCEPENDERECO_ENTREGA"] !== null)){
            
            $paramsValida = array(
                "CDFILIAL"      => $row["CDFILIAL"],
                "IDTPENDEFILI"  => "T",
            );

            $valida = $this->LocalAtendFactory->verificarExisteEndeFili($paramsValida);

            if ($valida == 0) {
                $isNew = true;
            }
            else {
                $isNew = false;
            };

            $entrega = array(
                "__is_new"      => $isNew,
                "CDPAIS"        => $row["CDPAIS_ENTREGA"],
                "SGESTADO"      => $row["SGESTADO_ENTREGA"],
                "CDMUNICIPIO"   => $row["CDMUNICIPIO_ENTREGA"],
                "DSENDEFILISEP" => $row["DSENDERECO_ENTREGA"],
                "NRENDEFILISEP" => $row["NRIMOVELENDERECO_ENTREGA"],
                "NMBAIRFILI"    => $row["NMBAIRROENDERECO_ENTREGA"],
                "NRCEPFILI"     => $row["NRCEPENDERECO_ENTREGA"],
                "DSCOMPENDFIL"  => $row["DSCOMPLEENDERECO_ENTREGA"],
                "IDTPENDEFILI"  => "T",
            );

            $this->integracoesTeknisaService->enviarDadosIntegracaoErpFilialEnde($url, array_merge($dadosIntegracao, $entrega), $email, $senha);
        }
    }

    public function getUrlIntegracaoErpFilial() {
        $nrorg = $this->environment->getNrOrg();
        $params = ["NRORG" => $nrorg];
        $dataset = $this->parametrizacaoServicesFactory->getUrlIntegracaoErpFilial($params);
        return $dataset[0]["URLINTEGRACAOERPFILIAL"];
    }

    public function getDadosOperador() {
        $cdoperador = $this->environment->getCdOperador();
        $nrorg = $this->environment->getNrOrg();

        $params = [
            "CDOPERADOR" => $cdoperador,
            "NRORG"      => $nrorg
        ];
        $dataset = $this->LocalAtendFactory->getDadosOperador($params);
        return $dataset[0];
    }

    public function getFilial($cdfilial) {
        $params = [
            "CDFILIAL" => $cdfilial
        ];
        
        $dataset = $this->LocalAtendFactory->getFilial($params);
        return count($dataset) > 0 ? $dataset[0] : null;
    }

    public function atualizarFilialLocalAtendimento($cdfilial, $nrestruturam) {
        $params = [
            "CDFILIAL"     => $cdfilial,
            "NRESTRUTURAM" => $nrestruturam
        ];

        $this->LocalAtendFactory->atualizarFilialLocalAtendimento($params);
    }

    /**
     * Valida se a filial já está relacionada a um outro local de atendimento
     */
    public function verificarFilialRepetidaOutroLocal($cdfilial, $nrestruturam) {
        $params = [
            "CDFILIAL"     => $cdfilial,
            "NRESTRUTURAM" => $nrestruturam
        ];

        $dataset = $this->LocalAtendFactory->getQtdLocalAtendimentoPorFilial($params);
        if ((int)$dataset[0]["QTD"] > 0) {
            throw new \Exception("FILIAL_VINCULADA_OUTRO_LOCAL");
        }
    }

    public function findEnderecos($parametros) {
        $param = array(
            "NRPARCNEGOCIO" => $parametros["NRPARCNEGOCIO"]
        );
        
        $dataset = $this->LocalAtendFactory->findEnderecos($param);
        return $dataset;
    }

    public function preparaDadosEnderecos($parametros) {
        $principal = array();
        $cobranca = array();
        $entrega = array();
        $comercial = array();
       
            foreach ($parametros['row']['ENDERECOS'] as $chave => $valor) {
                if (strpos($chave, '_principal') !== false) {
                    $chave = str_replace('_principal', '', $chave);
                    $principal[$chave] = $valor;
                    $principal["NRPARCNEGOCIO"] = $parametros['row']['NRPARCNEGOCIO'];
                    $retorno = $this->verificarSeExisteCadastroEndereco($parametros['row']['NRPARCNEGOCIO'], "PRINCIPAL");
                    $principal["NRENDERECOPARC"] = $retorno["NRENDERECOPARC"];
                    $principal["CDTIPOENDERECO"] = "PRINCIPAL";
                    ($retorno["NOVO"]) ? $principal["insert"] = true : $principal["insert"] = false;

                } elseif (strpos($chave, '_COBRANCA') !== false) {
                    $chave = str_replace('_COBRANCA', '', $chave);
                    $cobranca[$chave] = $valor;
                    $cobranca["NRPARCNEGOCIO"] = $parametros['row']['NRPARCNEGOCIO'];
                    $retorno = $this->verificarSeExisteCadastroEndereco($parametros['row']['NRPARCNEGOCIO'], "COBRANCA");
                    $cobranca["NRENDERECOPARC"] = $retorno["NRENDERECOPARC"];
                    $cobranca["CDTIPOENDERECO"] = "COBRANCA";
                    ($retorno["NOVO"]) ? $cobranca["insert"] = true : $cobranca["insert"] = false;

                } elseif (strpos($chave, '_ENTREGA') !== false){
                    $chave = str_replace('_ENTREGA', '', $chave);
                    $entrega[$chave] = $valor;
                    $entrega["NRPARCNEGOCIO"] = $parametros['row']['NRPARCNEGOCIO'];
                    $retorno = $this->verificarSeExisteCadastroEndereco($parametros['row']['NRPARCNEGOCIO'], "ENTREGA");
                    $entrega["NRENDERECOPARC"] = $retorno["NRENDERECOPARC"];
                    $entrega["CDTIPOENDERECO"] = "ENTREGA";
                    ($retorno["NOVO"]) ? $entrega["insert"] = true : $entrega["insert"] = false;
                    
                } else if(strpos($chave, '_EXTERIOR') !== false) {
                    $chave = str_replace('_EXTERIOR', '', $chave);
                    $comercial[$chave] = $valor;
                    $comercial["NRPARCNEGOCIO"] = $parametros['row']['NRPARCNEGOCIO'];
                    $retorno = $this->verificarSeExisteCadastroEndereco($parametros['row']['NRPARCNEGOCIO'], "COMERCIAL");
                    $comercial["NRENDERECOPARC"] = $retorno["NRENDERECOPARC"];
                    $comercial["CDTIPOENDERECO"] = "EXTERIOR";
                    ($retorno["NOVO"]) ? $comercial["insert"] = true : $comercial["insert"] = false;
                }
            }

            $enderecos = array($principal, $cobranca, $entrega, $comercial);
            foreach ($enderecos as $values) {
                $this->inserirAtualizarEnredecos($values);
            }
    }

    public function inserirAtualizarEnredecos($param) {
       
        $this->connection->beginTransaction();
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        try {
            
            $params = array(
                "NRENDERECOPARC"   => isset($param["NRENDERECOPARC"]) ? $param["NRENDERECOPARC"] : null,
                "NRPARCNEGOCIO"    => isset($param['NRPARCNEGOCIO']) ? $param['NRPARCNEGOCIO'] : null,
                "CDTIPOENDERECO"   => isset($param["CDTIPOENDERECO"]) ? $param["CDTIPOENDERECO"] : null,
                "NRORG"            => $nrorg,
                "IDATIVO"          => "S",
                "NRORGINCLUSAO"    => $nrorg,
                "CDOPERULTATU"     => $cdoperador,
                "CDPAIS"           => isset($param["CDPAIS"]) ? $param["CDPAIS"] : null,
                "SGESTADO"         => isset($param["SGESTADO"]) ? $param["SGESTADO"] : null,
                "CDMUNICIPIO"      => isset($param["CDMUNICIPIO"]) ? $param["CDMUNICIPIO"] : null,
                "CDBAIRRO"         => isset($param["CDBAIRRO"]) ?  $param["CDBAIRRO"] : null,
                "NMBAIRROENDERECO" => isset($param["NMBAIRROENDERECO"]) ? $param["NMBAIRROENDERECO"] : null,
                "DSCOMPLEENDERECO" => isset($param["DSCOMPLEENDERECO"]) ? $param["DSCOMPLEENDERECO"] : null ,
                "NRCEPENDERECO"    => isset($param["NRCEPENDERECO"]) ? str_replace(array(".", "-"), "", $param["NRCEPENDERECO"]) : null,
                "DSENDERECO"       => isset($param["DSENDERECO"]) ? $param["DSENDERECO"] : null,
                "NRIMOVELENDERECO" => isset($param["NRIMOVELENDERECO"]) ? $param["NRIMOVELENDERECO"] : null,
                "DSREFERENCIAENDE" => isset($param["DSREFERENCIAENDE"]) ? $param["DSREFERENCIAENDE"] : null,
                );
                
                if(isset($param["insert"]) && $param["insert"]) {
                     $this->LocalAtendFactory->inserirEnderecos($params);
                } else {
                    $this->LocalAtendFactory->atualizarEnderecos($params);
                }
                
            $this->connection->commit();
        }
        catch (\Exception $e) {
            throw new \Exception("Falha ao atualizar o endereço" . $e->getMessage());
        }
        
    }

    public function verificarSeExisteCadastroEndereco($nrparcneg, $tipo){
        $nrorg = $this->environment->getNrOrg();
        $retorno = $this->LocalAtendFactory->verificarSeExisteCadastroEndereco($nrparcneg, $tipo);
        
        if(isset($retorno[0])){
            $retornoData = array(
                "NRENDERECOPARC" => $retorno[0]["NRENDERECOPARC"],
                "NOVO" => false,
            );
        } else {
            $retornoData = array(
                "NRENDERECOPARC" => $this->newCode->getCodeByTableName("ENDERECOPARC", $nrorg, 1, 10, true),
                "NOVO" => true,
            );
        }
       return $retornoData;
    }

}

?>
